# -*- coding: utf-8 -*-
# Automatically generated - don't edit.
# Use `python setup.py update_constants` to update it.

MB_ATTRIBUTES = {
    'DB:cover_art_archive.art_type/name:001': 'Front',
    'DB:cover_art_archive.art_type/name:002': 'Back',
    'DB:cover_art_archive.art_type/name:003': 'Booklet',
    'DB:cover_art_archive.art_type/name:004': 'Medium',
    'DB:cover_art_archive.art_type/name:005': 'Obi',
    'DB:cover_art_archive.art_type/name:006': 'Spine',
    'DB:cover_art_archive.art_type/name:007': 'Track',
    'DB:cover_art_archive.art_type/name:008': 'Other',
    'DB:cover_art_archive.art_type/name:009': 'Tray',
    'DB:cover_art_archive.art_type/name:010': 'Sticker',
    'DB:cover_art_archive.art_type/name:011': 'Poster',
    'DB:cover_art_archive.art_type/name:012': 'Liner',
    'DB:cover_art_archive.art_type/name:013': 'Watermark',
    'DB:cover_art_archive.art_type/name:014': 'Raw/Unedited',
    'DB:medium_format/name:001': 'CD',
    'DB:medium_format/name:002': 'DVD',
    'DB:medium_format/name:003': 'SACD',
    'DB:medium_format/name:004': 'DualDisc',
    'DB:medium_format/name:005': 'LaserDisc',
    'DB:medium_format/name:006': 'MiniDisc',
    'DB:medium_format/name:007': 'Vinyl',
    'DB:medium_format/name:008': 'Cassette',
    'DB:medium_format/name:009': 'Cartridge',
    'DB:medium_format/name:010': 'Reel-to-reel',
    'DB:medium_format/name:011': 'DAT',
    'DB:medium_format/name:012': 'Digital Media',
    'DB:medium_format/name:013': 'Other',
    'DB:medium_format/name:014': 'Wax Cylinder',
    'DB:medium_format/name:015': 'Piano Roll',
    'DB:medium_format/name:016': 'DCC',
    'DB:medium_format/name:017': 'HD-DVD',
    'DB:medium_format/name:018': 'DVD-Audio',
    'DB:medium_format/name:019': 'DVD-Video',
    'DB:medium_format/name:020': 'Blu-ray',
    'DB:medium_format/name:021': 'VHS',
    'DB:medium_format/name:022': 'VCD',
    'DB:medium_format/name:023': 'SVCD',
    'DB:medium_format/name:024': 'Betamax',
    'DB:medium_format/name:025': 'HDCD',
    'DB:medium_format/name:026': 'USB Flash Drive',
    'DB:medium_format/name:027': 'slotMusic',
    'DB:medium_format/name:028': 'UMD',
    'DB:medium_format/name:029': '7" Vinyl',
    'DB:medium_format/name:030': '10" Vinyl',
    'DB:medium_format/name:031': '12" Vinyl',
    'DB:medium_format/name:033': 'CD-R',
    'DB:medium_format/name:034': '8cm CD',
    'DB:medium_format/name:035': 'Blu-spec CD',
    'DB:medium_format/name:036': 'SHM-CD',
    'DB:medium_format/name:037': 'HQCD',
    'DB:medium_format/name:038': 'Hybrid SACD',
    'DB:medium_format/name:039': 'CD+G',
    'DB:medium_format/name:040': '8cm CD+G',
    'DB:medium_format/name:041': 'CDV',
    'DB:medium_format/name:042': 'Enhanced CD',
    'DB:medium_format/name:043': 'Data CD',
    'DB:medium_format/name:044': 'DTS CD',
    'DB:medium_format/name:045': 'Playbutton',
    'DB:medium_format/name:046': 'Music Card',
    'DB:medium_format/name:047': 'DVDplus',
    'DB:medium_format/name:048': 'VinylDisc',
    'DB:medium_format/name:049': '3.5" Floppy Disk',
    'DB:medium_format/name:050': 'Edison Diamond Disc',
    'DB:medium_format/name:051': 'Flexi-disc',
    'DB:medium_format/name:052': '7" Flexi-disc',
    'DB:medium_format/name:053': 'Shellac',
    'DB:medium_format/name:054': '10" Shellac',
    'DB:medium_format/name:055': '12" Shellac',
    'DB:medium_format/name:056': '7" Shellac',
    'DB:medium_format/name:057': 'SHM-SACD',
    'DB:medium_format/name:058': 'Pathé disc',
    'DB:medium_format/name:059': 'VHD',
    'DB:medium_format/name:060': 'CED',
    'DB:medium_format/name:061': 'Copy Control CD',
    'DB:medium_format/name:062': 'SD Card',
    'DB:medium_format/name:063': 'Hybrid SACD (CD layer)',
    'DB:medium_format/name:064': 'Hybrid SACD (SACD layer)',
    'DB:medium_format/name:065': 'DualDisc (DVD-Audio side)',
    'DB:medium_format/name:066': 'DualDisc (DVD-Video side)',
    'DB:medium_format/name:067': 'DualDisc (CD side)',
    'DB:medium_format/name:068': 'DVDplus (DVD-Audio side)',
    'DB:medium_format/name:069': 'DVDplus (DVD-Video side)',
    'DB:medium_format/name:070': 'DVDplus (CD side)',
    'DB:medium_format/name:071': '8" LaserDisc',
    'DB:medium_format/name:072': '12" LaserDisc',
    'DB:medium_format/name:073': 'Phonograph record',
    'DB:medium_format/name:074': 'PlayTape',
    'DB:medium_format/name:075': 'HiPac',
    'DB:medium_format/name:076': 'Floppy Disk',
    'DB:medium_format/name:077': 'Zip Disk',
    'DB:medium_format/name:078': '8-Track Cartridge',
    'DB:medium_format/name:079': 'Blu-ray-R',
    'DB:medium_format/name:080': 'VinylDisc (DVD side)',
    'DB:medium_format/name:081': 'VinylDisc (Vinyl side)',
    'DB:medium_format/name:082': 'VinylDisc (CD side)',
    'DB:medium_format/name:083': 'Microcassette',
    'DB:release_group_primary_type/name:001': 'Album',
    'DB:release_group_primary_type/name:002': 'Single',
    'DB:release_group_primary_type/name:003': 'EP',
    'DB:release_group_primary_type/name:011': 'Other',
    'DB:release_group_primary_type/name:012': 'Broadcast',
    'DB:release_group_secondary_type/name:001': 'Compilation',
    'DB:release_group_secondary_type/name:002': 'Soundtrack',
    'DB:release_group_secondary_type/name:003': 'Spokenword',
    'DB:release_group_secondary_type/name:004': 'Interview',
    'DB:release_group_secondary_type/name:005': 'Audiobook',
    'DB:release_group_secondary_type/name:006': 'Live',
    'DB:release_group_secondary_type/name:007': 'Remix',
    'DB:release_group_secondary_type/name:008': 'DJ-mix',
    'DB:release_group_secondary_type/name:009': 'Mixtape/Street',
    'DB:release_group_secondary_type/name:010': 'Demo',
    'DB:release_group_secondary_type/name:011': 'Audio drama',
    'DB:release_status/name:001': 'Official',
    'DB:release_status/name:002': 'Promotion',
    'DB:release_status/name:003': 'Bootleg',
    'DB:release_status/name:004': 'Pseudo-Release',
}
